@@ -1,4 +1,4 @@
-/* xxxxxx -- an diagram creation/manipulation program
+/* Dia -- an diagram creation/manipulation program
  * Copyright (C) 1998 Alexander Larsson
  *
  * This program is free software; you can redistribute it and/or modify
@@ -19,55 +19,56 @@
 #include <stdlib.h>
 #include <math.h>
 
+#include "config.h"
+#include "intl.h"
 #include "grid.h"
+#include "preferences.h"
 
 void
-grid_draw(DDisplay *ddisp)
+grid_draw(DDisplay *ddisp, Rectangle *update)
 {
   Grid *grid = &ddisp->grid;
-
-  if (!grid->gc) {
-    Color col;
-    GdkColor gdkcol;
-    
-    grid->gc = gdk_gc_new(ddisp->pixmap);
-
-    col.red = 0.5; col.green = 0.5; col.blue = 0.5;
-    color_convert(&col, &gdkcol);
-    
-    gdk_gc_set_foreground(grid->gc, &gdkcol);
-  }
+  Renderer *renderer = &ddisp->renderer->renderer;
+  int width = ddisp->renderer->renderer.pixel_width;
+  int height = ddisp->renderer->renderer.pixel_height;
 
   if ( (ddisplay_transform_length(ddisp, grid->width_x) <= 1.0) ||
-       (ddisplay_transform_length(ddisp, grid->width_y) <= 1.0) )
+       (ddisplay_transform_length(ddisp, grid->width_y) <= 1.0) ) {
     return;
-  
-
-
-  
-  gdk_gc_set_clip_region(grid->gc, ddisp->clip_region);
+  }
   
   if (grid->visible) {
     real pos;
     int x,y;
+
+    (renderer->ops->set_linewidth)(renderer, 0.0);
+    if (prefs.grid.solid) {
+      (renderer->ops->set_linestyle)(renderer, LINESTYLE_SOLID);
+    } else {
+      (renderer->ops->set_dashlength)(renderer,
+      		    ddisplay_untransform_length(ddisp, 31));
+      (renderer->ops->set_linestyle)(renderer, LINESTYLE_DOTTED);
+    }
     
     /* Vertical lines: */
-    pos = ceil( ddisp->visible.left / grid->width_x )*grid->width_x;
-    while (pos <= ddisp->visible.right) {
+    pos = ceil( update->left / grid->width_x )*grid->width_x;
+    while (pos <= update->right) {
       ddisplay_transform_coords(ddisp, pos,0,&x,&y);
-      gdk_draw_line(ddisp->pixmap, grid->gc,
-		    x, 0,
-		    x, ddisp->height);
+      (renderer->interactive_ops->draw_pixel_line)(renderer,
+						   x, 0,
+						   x, height,
+						   &prefs.grid.colour);
       pos += grid->width_x;
     }
 
     /* Horizontal lines: */
-    pos = ceil( ddisp->visible.top / grid->width_y )*grid->width_y;
-    while (pos <= ddisp->visible.bottom) {
+    pos = ceil( update->top / grid->width_y )*grid->width_y;
+    while (pos <= update->bottom) {
       ddisplay_transform_coords(ddisp, 0,pos,&x,&y);
-      gdk_draw_line(ddisp->pixmap, grid->gc,
-		    0, y,
-		    ddisp->width, y);
+      (renderer->interactive_ops->draw_pixel_line)(renderer,
+						   0, y,
+						   width, y,
+						   &prefs.grid.colour);
       pos += grid->width_y;
     }
   }
@@ -143,8 +144,8 @@ void grid_show_dialog(Grid *grid, DDisplay *ddisp)
     char buffer[32];
     
     grid->dialog = dialog = gtk_window_new(GTK_WINDOW_DIALOG);
-    gtk_window_set_title (GTK_WINDOW (dialog), "Grid options");
-    gtk_container_border_width (GTK_CONTAINER (dialog), 5);
+    gtk_window_set_title (GTK_WINDOW (dialog), _("Grid options"));
+    gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
     gtk_signal_connect (GTK_OBJECT (dialog), "delete_event",
 			GTK_SIGNAL_FUNC(gtk_widget_hide), NULL);
 
@@ -152,7 +153,7 @@ void grid_show_dialog(Grid *grid, DDisplay *ddisp)
     gtk_container_add(GTK_CONTAINER(dialog), vbox);
     
     hbox = gtk_hbox_new(FALSE, 5);
-    label = gtk_label_new("Grid x size:");
+    label = gtk_label_new(_("Grid x size:"));
     grid->entry_x = entry = gtk_entry_new();
     gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
     gtk_box_pack_start (GTK_BOX (hbox), entry, TRUE, TRUE, 0);
@@ -170,7 +171,7 @@ void grid_show_dialog(Grid *grid, DDisplay *ddisp)
 			GTK_SIGNAL_FUNC (grid_x_update), ddisp);
 
     hbox = gtk_hbox_new(FALSE, 5);
-    label = gtk_label_new("Grid y size:");
+    label = gtk_label_new(_("Grid y size:"));
     grid->entry_y = entry = gtk_entry_new();
     gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
     gtk_box_pack_start (GTK_BOX (hbox), entry, TRUE, TRUE, 0);
