@@ -1,4 +1,4 @@
-/* xxxxxx -- an diagram creation/manipulation program
+/* Dia -- an diagram creation/manipulation program
  * Copyright (C) 1998 Alexander Larsson
  *
  * This program is free software; you can redistribute it and/or modify
@@ -30,6 +30,8 @@ struct _RendererEPS {
   Renderer renderer;
 
   FILE *file;
+  int is_ps;
+  int pagenum;
 
   LineStyle saved_line_style;
   real dash_length;
@@ -37,7 +39,9 @@ struct _RendererEPS {
 };
 
 extern RendererEPS *new_eps_renderer(Diagram *dia, char *filename);
-extern void close_eps_renderer(RendererEPS *renderer);
+extern RendererEPS *new_psprint_renderer(Diagram *dia, FILE *file);
+extern void destroy_eps_renderer(RendererEPS *renderer);
+
 #endif /* RENDER_EPS_H */
 
 
