@@ -1,4 +1,4 @@
-/* xxxxxx -- an diagram creation/manipulation program
+/* Dia -- an diagram creation/manipulation program
  * Copyright (C) 1998 Alexander Larsson
  *
  * This program is free software; you can redistribute it and/or modify
@@ -20,15 +20,14 @@
 #include <math.h>
 #include <string.h>
 
+#include "config.h"
+#include "intl.h"
 #include "render.h"
 #include "attributes.h"
-#include "files.h"
-#include "sheet.h"
 
 #include "class.h"
 
 #include "pixmaps/umlclass.xpm"
-#include "pixmaps/umlclass_template.xpm"
 
 #define UMLCLASS_BORDER 0.1
 #define UMLCLASS_UNDERLINEWIDTH 0.05
@@ -37,7 +36,7 @@ static real umlclass_distance_from(UMLClass *umlclass, Point *point);
 static void umlclass_select(UMLClass *umlclass, Point *clicked_point,
 			    Renderer *interactive_renderer);
 static void umlclass_move_handle(UMLClass *umlclass, Handle *handle,
-				 Point *to, HandleMoveReason reason);
+				 Point *to, HandleMoveReason reason, ModifierKeys modifiers);
 static void umlclass_move(UMLClass *umlclass, Point *to);
 static void umlclass_draw(UMLClass *umlclass, Renderer *renderer);
 static Object *umlclass_create(Point *startpoint,
@@ -47,8 +46,10 @@ static Object *umlclass_create(Point *startpoint,
 static void umlclass_destroy(UMLClass *umlclass);
 static Object *umlclass_copy(UMLClass *umlclass);
 
-static void umlclass_save(UMLClass *umlclass, int fd);
-static Object *umlclass_load(int fd, int version);
+static void umlclass_save(UMLClass *umlclass, ObjectNode obj_node,
+			  const char *filename);
+static Object *umlclass_load(ObjectNode obj_node, int version,
+			     const char *filename);
 
 static ObjectTypeOps umlclass_type_ops =
 {
@@ -66,34 +67,17 @@ ObjectType umlclass_type =
   &umlclass_type_ops       /* ops */
 };
 
-SheetObject umlclass_sheetobj =
-{
-  &umlclass_type,             /* type */
-  "Create a class",           /* description */
-  (char **) umlclass_xpm,     /* pixmap */
-
-  GINT_TO_POINTER(0)          /* user_data */
-};
-
-SheetObject umlclass_template_sheetobj =
-{
-  &umlclass_type,             /* type */
-  "Create a template class",  /* description */
-  (char **) umlclass_template_xpm,     /* pixmap */
-
-  GINT_TO_POINTER(1)          /* user_data */
-};
-
 static ObjectOps umlclass_ops = {
-  (DestroyFunc)        umlclass_destroy,
-  (DrawFunc)           umlclass_draw,
-  (DistanceFunc)       umlclass_distance_from,
-  (SelectFunc)         umlclass_select,
-  (CopyFunc)           umlclass_copy,
-  (MoveFunc)           umlclass_move,
-  (MoveHandleFunc)     umlclass_move_handle,
-  (ShowPropertiesFunc) umlclass_show_properties,
-  (IsEmptyFunc)        object_return_false
+  (DestroyFunc)         umlclass_destroy,
+  (DrawFunc)            umlclass_draw,
+  (DistanceFunc)        umlclass_distance_from,
+  (SelectFunc)          umlclass_select,
+  (CopyFunc)            umlclass_copy,
+  (MoveFunc)            umlclass_move,
+  (MoveHandleFunc)      umlclass_move_handle,
+  (GetPropertiesFunc)   umlclass_get_properties,
+  (ApplyPropertiesFunc) umlclass_apply_properties,
+  (ObjectMenuFunc)      NULL
 };
 
 static real
@@ -112,7 +96,7 @@ umlclass_select(UMLClass *umlclass, Point *clicked_point,
 
 static void
 umlclass_move_handle(UMLClass *umlclass, Handle *handle,
-		     Point *to, HandleMoveReason reason)
+		     Point *to, HandleMoveReason reason, ModifierKeys modifiers)
 {
   assert(umlclass!=NULL);
   assert(handle!=NULL);
@@ -166,9 +150,10 @@ umlclass_draw(UMLClass *umlclass, Renderer *renderer)
   /* stereotype: */
   font = umlclass->normal_font;
   p.x = x + elem->width / 2.0;
-  p.y = p1.y + 0.1 + umlclass->font_ascent;
+  p.y = p1.y;
 
   if (umlclass->stereotype != NULL) {
+    p.y += 0.1 + umlclass->font_ascent;
     renderer->ops->set_font(renderer, font, umlclass->font_height);
     renderer->ops->draw_string(renderer,
 			       umlclass->stereotype_string,
@@ -273,12 +258,13 @@ umlclass_draw(UMLClass *umlclass, Renderer *renderer)
 				   &color_black);
 
 	if (op->class_scope) {
-	  p.y += umlclass->font_height*0.1;
-	  p1 = p;
-	  p1.x += font_string_width(umlclass->operations_strings[i],
+	  p1 = p; 
+	  p1.y += umlclass->font_height*0.1;
+	  p3 = p1;
+	  p3.x += font_string_width(umlclass->operations_strings[i],
 				    font, umlclass->font_height);
 	  renderer->ops->set_linewidth(renderer, UMLCLASS_UNDERLINEWIDTH);
-	  renderer->ops->draw_line(renderer, &p, &p1,  &color_black);
+	  renderer->ops->draw_line(renderer, &p1, &p3,  &color_black);
 	  renderer->ops->set_linewidth(renderer, UMLCLASS_BORDER);
 	}
 
@@ -338,24 +324,62 @@ umlclass_update_data(UMLClass *umlclass)
 {
   Element *elem = &umlclass->element;
   Object *obj = (Object *)umlclass;
+  real x,y;
+  GList *list;
+
+  x = elem->corner.x;
+  y = elem->corner.y;
   
   /* Update connections: */
   umlclass->connections[0].pos = elem->corner;
-  umlclass->connections[1].pos.x = elem->corner.x + elem->width / 2.0;
-  umlclass->connections[1].pos.y = elem->corner.y;
-  umlclass->connections[2].pos.x = elem->corner.x + elem->width;
-  umlclass->connections[2].pos.y = elem->corner.y;
-  umlclass->connections[3].pos.x = elem->corner.x;
-  umlclass->connections[3].pos.y = elem->corner.y + elem->height / 2.0;
-  umlclass->connections[4].pos.x = elem->corner.x + elem->width;
-  umlclass->connections[4].pos.y = elem->corner.y + elem->height / 2.0;
-  umlclass->connections[5].pos.x = elem->corner.x;
-  umlclass->connections[5].pos.y = elem->corner.y + elem->height;
-  umlclass->connections[6].pos.x = elem->corner.x + elem->width / 2.0;
-  umlclass->connections[6].pos.y = elem->corner.y + elem->height;
-  umlclass->connections[7].pos.x = elem->corner.x + elem->width;
-  umlclass->connections[7].pos.y = elem->corner.y + elem->height;
+  umlclass->connections[1].pos.x = x + elem->width / 2.0;
+  umlclass->connections[1].pos.y = y;
+  umlclass->connections[2].pos.x = x + elem->width;
+  umlclass->connections[2].pos.y = y;
+  umlclass->connections[3].pos.x = x;
+  umlclass->connections[3].pos.y = y + umlclass->namebox_height/2.0;
+  umlclass->connections[4].pos.x = x + elem->width;
+  umlclass->connections[4].pos.y = y + umlclass->namebox_height/2.0;
+  umlclass->connections[5].pos.x = x;
+  umlclass->connections[5].pos.y = y + elem->height;
+  umlclass->connections[6].pos.x = x + elem->width / 2.0;
+  umlclass->connections[6].pos.y = y + elem->height;
+  umlclass->connections[7].pos.x = x + elem->width;
+  umlclass->connections[7].pos.y = y + elem->height;
 
+  y += umlclass->namebox_height + 0.1 + umlclass->font_height/2;
+
+  list = umlclass->attributes;
+  while (list != NULL) {
+    UMLAttribute *attr = (UMLAttribute *)list->data;
+
+    attr->left_connection->pos.x = x;
+    attr->left_connection->pos.y = y;
+    attr->right_connection->pos.x = x + elem->width;
+    attr->right_connection->pos.y = y;
+
+    y += umlclass->font_height;
+
+    list = g_list_next(list);
+  }
+
+  y = elem->corner.y + umlclass->namebox_height +
+    umlclass->attributesbox_height + 0.1 + umlclass->font_height/2;
+    
+  list = umlclass->operations;
+  while (list != NULL) {
+    UMLOperation *op = (UMLOperation *)list->data;
+
+    op->left_connection->pos.x = x;
+    op->left_connection->pos.y = y;
+    op->right_connection->pos.x = x + elem->width;
+    op->right_connection->pos.y = y;
+
+    y += umlclass->font_height;
+
+    list = g_list_next(list);
+  }
+  
   element_update_boundingbox(elem);
   /* fix boundingumlclass for line_width: */
   obj->bounding_box.top -= UMLCLASS_BORDER/2.0;
@@ -398,12 +422,12 @@ umlclass_calculate_data(UMLClass *umlclass)
 				 umlclass->classname_font,
 				 umlclass->classname_font_height);
 
-  umlclass->namebox_height =
-    font_height + umlclass->classname_font_height + 2*0.1;
+  umlclass->namebox_height = umlclass->classname_font_height + 4*0.1;
   if (umlclass->stereotype_string != NULL) {
     g_free(umlclass->stereotype_string);
   }
   if (umlclass->stereotype != NULL) {
+    umlclass->namebox_height += font_height;
     umlclass->stereotype_string =
       g_malloc(sizeof(char)*(strlen(umlclass->stereotype)+2+1));
     
@@ -435,7 +459,10 @@ umlclass_calculate_data(UMLClass *umlclass)
   if ((umlclass->attributesbox_height<0.4) ||
       umlclass->suppress_attributes )
       umlclass->attributesbox_height = 0.4;
-  
+
+  if (!umlclass->visible_attributes )
+    umlclass->attributesbox_height = 0.0;
+
   umlclass->attributes_strings = NULL;
   if (umlclass->num_attributes != 0) {
     umlclass->attributes_strings =
@@ -545,11 +572,11 @@ static void
 fill_in_fontdata(UMLClass *umlclass)
 {
   umlclass->font_height = 0.8;
-  umlclass->classname_font_height = 1.2;
+  umlclass->classname_font_height = 1.0;
   umlclass->normal_font = font_getfont("Courier");
   umlclass->abstract_font = font_getfont("Courier-Oblique");
-  umlclass->classname_font = font_getfont("Courier-Bold");
-  umlclass->abstract_classname_font = font_getfont("Courier-BoldOblique");
+  umlclass->classname_font = font_getfont("Helvetica-Bold");
+  umlclass->abstract_classname_font = font_getfont("Helvetica-BoldOblique");
 }
 
 static Object *
@@ -573,12 +600,12 @@ umlclass_create(Point *startpoint,
 
   elem->corner = *startpoint;
 
-  element_init(elem, 8, 8);
+  element_init(elem, 8, 8); /* No attribs or ops => 0 extra connectionpoints. */
 
   umlclass->properties_dialog = NULL;
   fill_in_fontdata(umlclass);
 
-  umlclass->name = strdup("Class");
+  umlclass->name = strdup(_("Class"));
   umlclass->stereotype = NULL;
   
   umlclass->abstract = FALSE;
@@ -628,6 +655,8 @@ umlclass_destroy(UMLClass *umlclass)
   UMLOperation *op;
   UMLFormalParameter *param;
   
+  element_destroy(&umlclass->element);
+  
   g_free(umlclass->name);
   if (umlclass->stereotype != NULL)
     g_free(umlclass->stereotype);
@@ -635,6 +664,8 @@ umlclass_destroy(UMLClass *umlclass)
   list = umlclass->attributes;
   while (list != NULL) {
     attr = (UMLAttribute *)list->data;
+    g_free(attr->left_connection);
+    g_free(attr->right_connection);
     uml_attribute_destroy(attr);
     list = g_list_next(list);
   }
@@ -643,6 +674,8 @@ umlclass_destroy(UMLClass *umlclass)
   list = umlclass->operations;
   while (list != NULL) {
     op = (UMLOperation *)list->data;
+    g_free(op->left_connection);
+    g_free(op->right_connection);
     uml_operation_destroy(op);
     list = g_list_next(list);
   }
@@ -683,10 +716,9 @@ umlclass_destroy(UMLClass *umlclass)
 
   if (umlclass->properties_dialog != NULL) {
     gtk_widget_destroy(umlclass->properties_dialog->dialog);
+    g_list_free(umlclass->properties_dialog->deleted_connections);
     g_free(umlclass->properties_dialog);
   }
-
-  element_destroy(&umlclass->element);
 }
 
 static Object *
@@ -698,7 +730,9 @@ umlclass_copy(UMLClass *umlclass)
   Object *newobj;
   GList *list;
   UMLAttribute *attr;
+  UMLAttribute *newattr;
   UMLOperation *op;
+  UMLOperation *newop;
   UMLFormalParameter *param;
   
   elem = &umlclass->element;
@@ -731,8 +765,20 @@ umlclass_copy(UMLClass *umlclass)
   list = umlclass->attributes;
   while (list != NULL) {
     attr = (UMLAttribute *)list->data;
+    newattr = uml_attribute_copy(attr);
+    
+    newattr->left_connection = g_new(ConnectionPoint,1);
+    *newattr->left_connection = *attr->left_connection;
+    newattr->left_connection->object = newobj;
+    newattr->left_connection->connected = NULL;
+    
+    newattr->right_connection = g_new(ConnectionPoint,1);
+    *newattr->right_connection = *attr->right_connection;
+    newattr->right_connection->object = newobj;
+    newattr->right_connection->connected = NULL;
+    
     newumlclass->attributes = g_list_prepend(newumlclass->attributes,
-					     uml_attribute_copy(attr));
+					     newattr);
     list = g_list_next(list);
   }
 
@@ -740,8 +786,19 @@ umlclass_copy(UMLClass *umlclass)
   list = umlclass->operations;
   while (list != NULL) {
     op = (UMLOperation *)list->data;
+    newop = uml_operation_copy(op);
+    newop->left_connection = g_new(ConnectionPoint,1);
+    *newop->left_connection = *op->left_connection;
+    newop->left_connection->object = newobj;
+    newop->left_connection->connected = NULL;
+
+    newop->right_connection = g_new(ConnectionPoint,1);
+    *newop->right_connection = *op->right_connection;
+    newop->right_connection->object = newobj;
+    newop->right_connection->connected = NULL;
+    
     newumlclass->operations = g_list_prepend(newumlclass->operations,
-					     uml_operation_copy(op));
+					     newop);
     list = g_list_next(list);
   }
 
@@ -774,6 +831,31 @@ umlclass_copy(UMLClass *umlclass)
 
   umlclass_calculate_data(newumlclass);
 
+  i = 8;
+  if ( (newumlclass->visible_attributes) &&
+       (!newumlclass->suppress_attributes)) {
+    list = newumlclass->attributes;
+    while (list != NULL) {
+      attr = (UMLAttribute *)list->data;
+      newobj->connections[i++] = attr->left_connection;
+      newobj->connections[i++] = attr->right_connection;
+      
+      list = g_list_next(list);
+    }
+  }
+  
+  if ( (newumlclass->visible_operations) &&
+       (!newumlclass->suppress_operations)) {
+    list = newumlclass->operations;
+    while (list != NULL) {
+      op = (UMLOperation *)list->data;
+      newobj->connections[i++] = op->left_connection;
+      newobj->connections[i++] = op->right_connection;
+      
+      list = g_list_next(list);
+    }
+  }
+
   umlclass_update_data(newumlclass);
   
   return (Object *)newumlclass;
@@ -781,54 +863,66 @@ umlclass_copy(UMLClass *umlclass)
 
 
 static void
-umlclass_save(UMLClass *umlclass, int fd)
+umlclass_save(UMLClass *umlclass, ObjectNode obj_node,
+	      const char *filename)
 {
   UMLAttribute *attr;
   UMLOperation *op;
   UMLFormalParameter *formal_param;
   GList *list;
+  AttributeNode attr_node;
   
-  element_save(&umlclass->element, fd);
+  element_save(&umlclass->element, obj_node);
 
   /* Class info: */
-  write_string(fd, umlclass->name);
-  write_string(fd, umlclass->stereotype);
-  write_int32(fd, umlclass->abstract);
-  write_int32(fd, umlclass->suppress_attributes);
-  write_int32(fd, umlclass->suppress_operations);
-  write_int32(fd, umlclass->visible_attributes);
-  write_int32(fd, umlclass->visible_operations);
+  data_add_string(new_attribute(obj_node, "name"),
+		  umlclass->name);
+  data_add_string(new_attribute(obj_node, "stereotype"),
+		  umlclass->stereotype);
+  data_add_boolean(new_attribute(obj_node, "abstract"),
+		   umlclass->abstract);
+  data_add_boolean(new_attribute(obj_node, "suppress_attributes"),
+		   umlclass->suppress_attributes);
+  data_add_boolean(new_attribute(obj_node, "suppress_operations"),
+		   umlclass->suppress_operations);
+  data_add_boolean(new_attribute(obj_node, "visible_attributes"),
+		   umlclass->visible_attributes);
+  data_add_boolean(new_attribute(obj_node, "visible_operations"),
+		   umlclass->visible_operations);
 
   /* Attribute info: */
-  write_int32(fd, g_list_length(umlclass->attributes));
+  attr_node = new_attribute(obj_node, "attributes");
   list = umlclass->attributes;
   while (list != NULL) {
     attr = (UMLAttribute *) list->data;
-    uml_attribute_write(fd, attr);
+    uml_attribute_write(attr_node, attr);
     list = g_list_next(list);
   }
-
+  
   /* Operations info: */
-  write_int32(fd, g_list_length(umlclass->operations));
+  attr_node = new_attribute(obj_node, "operations");
   list = umlclass->operations;
   while (list != NULL) {
     op = (UMLOperation *) list->data;
-    uml_operation_write(fd, op);
+    uml_operation_write(attr_node, op);
     list = g_list_next(list);
   }
 
   /* Template info: */
-  write_int32(fd, umlclass->template);
-  write_int32(fd, g_list_length(umlclass->formal_params));
+  data_add_boolean(new_attribute(obj_node, "template"),
+		   umlclass->template);
+  
+  attr_node = new_attribute(obj_node, "templates");
   list = umlclass->formal_params;
   while (list != NULL) {
     formal_param = (UMLFormalParameter *) list->data;
-    uml_formalparameter_write(fd, formal_param);
+    uml_formalparameter_write(attr_node, formal_param);
     list = g_list_next(list);
   }
 }
 
-static Object *umlclass_load(int fd, int version)
+static Object *umlclass_load(ObjectNode obj_node, int version,
+			     const char *filename)
 {
   UMLClass *umlclass;
   Element *elem;
@@ -836,8 +930,11 @@ static Object *umlclass_load(int fd, int version)
   UMLAttribute *attr;
   UMLOperation *op;
   UMLFormalParameter *formal_param;
+  AttributeNode attr_node;
+  DataNode composite;
   int i;
-  int num;
+  int num, num_attr, num_ops;
+  GList *list;
   
   umlclass = g_malloc(sizeof(UMLClass));
   elem = &umlclass->element;
@@ -846,43 +943,109 @@ static Object *umlclass_load(int fd, int version)
   obj->type = &umlclass_type;
   obj->ops = &umlclass_ops;
 
-  element_load(elem, fd);
+  element_load(elem, obj_node);
   
   /* Class info: */
-  umlclass->name = read_string(fd);
-  umlclass->stereotype = read_string(fd);
-  umlclass->abstract = read_int32(fd);
-  umlclass->suppress_attributes = read_int32(fd);
-  umlclass->suppress_operations = read_int32(fd);
-  umlclass->visible_attributes = read_int32(fd);
-  umlclass->visible_operations = read_int32(fd);
-
+  umlclass->name = NULL;
+  attr_node = object_find_attribute(obj_node, "name");
+  if (attr_node != NULL)
+    umlclass->name = data_string(attribute_first_data(attr_node));
+  
+  umlclass->stereotype = NULL;
+  attr_node = object_find_attribute(obj_node, "stereotype");
+  if (attr_node != NULL)
+    umlclass->stereotype = data_string(attribute_first_data(attr_node));
+  
+  umlclass->abstract = FALSE;
+  attr_node = object_find_attribute(obj_node, "abstract");
+  if (attr_node != NULL)
+    umlclass->abstract = data_boolean(attribute_first_data(attr_node));
+  
+  umlclass->suppress_attributes = FALSE;
+  attr_node = object_find_attribute(obj_node, "suppress_attributes");
+  if (attr_node != NULL)
+    umlclass->suppress_attributes = data_boolean(attribute_first_data(attr_node));
+  
+  umlclass->suppress_operations = FALSE;
+  attr_node = object_find_attribute(obj_node, "suppress_operations");
+  if (attr_node != NULL)
+    umlclass->suppress_operations = data_boolean(attribute_first_data(attr_node));
+  
+  umlclass->visible_attributes = FALSE;
+  attr_node = object_find_attribute(obj_node, "visible_attributes");
+  if (attr_node != NULL)
+    umlclass->visible_attributes = data_boolean(attribute_first_data(attr_node));
+  
+  umlclass->visible_operations = FALSE;
+  attr_node = object_find_attribute(obj_node, "visible_operations");
+  if (attr_node != NULL)
+    umlclass->visible_operations = data_boolean(attribute_first_data(attr_node));
+  
   /* Attribute info: */
-  num = read_int32(fd);
+  attr_node = object_find_attribute(obj_node, "attributes");
+  num_attr = num = attribute_num_data(attr_node);
+  composite = attribute_first_data(attr_node);
   umlclass->attributes = NULL;
   for (i=0;i<num;i++) {
-    attr = uml_attribute_read(fd);
+    attr = uml_attribute_read(composite);
+
+    attr->left_connection = g_new(ConnectionPoint,1);
+    attr->left_connection->object = obj;
+    attr->left_connection->connected = NULL;
+
+    attr->right_connection = g_new(ConnectionPoint,1);
+    attr->right_connection->object = obj;
+    attr->right_connection->connected = NULL;
+
     umlclass->attributes = g_list_append(umlclass->attributes, attr);
+    composite = data_next(composite);
   }
 
   /* Operations info: */
-  num = read_int32(fd);
+  attr_node = object_find_attribute(obj_node, "operations");
+  num_ops = num = attribute_num_data(attr_node);
+  composite = attribute_first_data(attr_node);
   umlclass->operations = NULL;
   for (i=0;i<num;i++) {
-    op = uml_operation_read(fd);
+    op = uml_operation_read(composite);
+
+    op->left_connection = g_new(ConnectionPoint,1);
+    op->left_connection->object = obj;
+    op->left_connection->connected = NULL;
+
+    op->right_connection = g_new(ConnectionPoint,1);
+    op->right_connection->object = obj;
+    op->right_connection->connected = NULL;
+    
     umlclass->operations = g_list_append(umlclass->operations, op);
+    composite = data_next(composite);
   }
 
-  /* Attribute info: */
-  umlclass->template = read_int32(fd);
-  num = read_int32(fd);
+  /* Template info: */
+  umlclass->template = FALSE;
+  attr_node = object_find_attribute(obj_node, "template");
+  if (attr_node != NULL)
+    umlclass->template = data_boolean(attribute_first_data(attr_node));
+
+  attr_node = object_find_attribute(obj_node, "templates");
+  num = attribute_num_data(attr_node);
+  composite = attribute_first_data(attr_node);
   umlclass->formal_params = NULL;
   for (i=0;i<num;i++) {
-    formal_param = uml_formalparameter_read(fd);
+    formal_param = uml_formalparameter_read(composite);
     umlclass->formal_params = g_list_append(umlclass->formal_params, formal_param);
+    composite = data_next(composite);
   }
 
-  element_init(elem, 8, 8);
+  if ( (!umlclass->visible_attributes) ||
+       (umlclass->suppress_attributes))
+    num_attr = 0;
+  
+  if ( (!umlclass->visible_operations) ||
+       (umlclass->suppress_operations))
+    num_ops = 0;
+  
+  element_init(elem, 8, 8 + num_attr*2 + num_ops*2);
 
   umlclass->properties_dialog = NULL;
   fill_in_fontdata(umlclass);
@@ -899,8 +1062,36 @@ static Object *umlclass_load(int fd, int version)
     umlclass->connections[i].object = obj;
     umlclass->connections[i].connected = NULL;
   }
+
+  i = 8;
+
+  if ( (umlclass->visible_attributes) &&
+       (!umlclass->suppress_attributes)) {
+    list = umlclass->attributes;
+    while (list != NULL) {
+      attr = (UMLAttribute *)list->data;
+      obj->connections[i++] = attr->left_connection;
+      obj->connections[i++] = attr->right_connection;
+      
+      list = g_list_next(list);
+    }
+  }
+  
+  if ( (umlclass->visible_operations) &&
+       (!umlclass->suppress_operations)) {
+    list = umlclass->operations;
+    while (list != NULL) {
+      op = (UMLOperation *)list->data;
+      obj->connections[i++] = op->left_connection;
+      obj->connections[i++] = op->right_connection;
+      
+      list = g_list_next(list);
+    }
+  }
+
   umlclass_update_data(umlclass);
 
+  
   for (i=0;i<8;i++) {
     obj->handles[i]->type = HANDLE_NON_MOVABLE;
   }
@@ -911,3 +1102,6 @@ static Object *umlclass_load(int fd, int version)
 
 
 
+
+
+
