@@ -1,4 +1,4 @@
-/* xxxxxx -- an diagram creation/manipulation program
+/* Dia -- an diagram creation/manipulation program
  * Copyright (C) 1998 Alexander Larsson
  *
  * This program is free software; you can redistribute it and/or modify
@@ -20,12 +20,12 @@
 #include <math.h>
 #include <string.h>
 
+#include "config.h"
+#include "intl.h"
 #include "object.h"
 #include "orth_conn.h"
 #include "render.h"
 #include "attributes.h"
-#include "files.h"
-#include "sheet.h"
 #include "arrows.h"
 
 #include "uml.h"
@@ -33,8 +33,15 @@
 #include "pixmaps/generalization.xpm"
 
 typedef struct _Generalization Generalization;
+typedef struct _GeneralizationState GeneralizationState;
 typedef struct _GeneralizationPropertiesDialog GeneralizationPropertiesDialog;
 
+struct _GeneralizationState {
+  ObjectState obj_state;
+
+  char *name;
+  char *stereotype; 
+};
 struct _Generalization {
   OrthConn orth;
 
@@ -53,9 +60,6 @@ struct _GeneralizationPropertiesDialog {
   
   GtkEntry *name;
   GtkEntry *stereotype;
-  
-  ObjectChangedFunc *changed_callback;
-  void *changed_callback_data;
 };
 
 #define GENERALIZATION_WIDTH 0.1
@@ -68,7 +72,7 @@ static real generalization_distance_from(Generalization *genlz, Point *point);
 static void generalization_select(Generalization *genlz, Point *clicked_point,
 			      Renderer *interactive_renderer);
 static void generalization_move_handle(Generalization *genlz, Handle *handle,
-				   Point *to, HandleMoveReason reason);
+				   Point *to, HandleMoveReason reason, ModifierKeys modifiers);
 static void generalization_move(Generalization *genlz, Point *to);
 static void generalization_draw(Generalization *genlz, Renderer *renderer);
 static Object *generalization_create(Point *startpoint,
@@ -77,12 +81,19 @@ static Object *generalization_create(Point *startpoint,
 				 Handle **handle2);
 static void generalization_destroy(Generalization *genlz);
 static Object *generalization_copy(Generalization *genlz);
-static void generalization_show_properties(Generalization *genlz,
-				       ObjectChangedFunc *changed_callback,
-				       void *changed_callback_data);
+static GtkWidget *generalization_get_properties(Generalization *genlz);
+static ObjectChange *generalization_apply_properties(Generalization *genlz);
+static DiaMenu *generalization_get_object_menu(Generalization *genlz,
+						Point *clickedpoint);
 
-static void generalization_save(Generalization *genlz, int fd);
-static Object *generalization_load(int fd, int version);
+static GeneralizationState *generalization_get_state(Generalization *genlz);
+static void generalization_set_state(Generalization *genlz,
+				     GeneralizationState *state);
+
+static void generalization_save(Generalization *genlz, ObjectNode obj_node,
+				const char *filename);
+static Object *generalization_load(ObjectNode obj_node, int version,
+				   const char *filename);
 
 static void generalization_update_data(Generalization *genlz);
 
@@ -102,26 +113,17 @@ ObjectType generalization_type =
   &generalization_type_ops       /* ops */
 };
 
-SheetObject generalization_sheetobj =
-{
-  &generalization_type,             /* type */
-  "Generalization, class inheritance.",
-                              /* description */
-  (char **) generalization_xpm,     /* pixmap */
-
-  NULL                        /* user_data */
-};
-
 static ObjectOps generalization_ops = {
-  (DestroyFunc)        generalization_destroy,
-  (DrawFunc)           generalization_draw,
-  (DistanceFunc)       generalization_distance_from,
-  (SelectFunc)         generalization_select,
-  (CopyFunc)           generalization_copy,
-  (MoveFunc)           generalization_move,
-  (MoveHandleFunc)     generalization_move_handle,
-  (ShowPropertiesFunc) generalization_show_properties,
-  (IsEmptyFunc)        object_return_false
+  (DestroyFunc)         generalization_destroy,
+  (DrawFunc)            generalization_draw,
+  (DistanceFunc)        generalization_distance_from,
+  (SelectFunc)          generalization_select,
+  (CopyFunc)            generalization_copy,
+  (MoveFunc)            generalization_move,
+  (MoveHandleFunc)      generalization_move_handle,
+  (GetPropertiesFunc)   generalization_get_properties,
+  (ApplyPropertiesFunc) generalization_apply_properties,
+  (ObjectMenuFunc)      generalization_get_object_menu
 };
 
 static real
@@ -140,7 +142,7 @@ generalization_select(Generalization *genlz, Point *clicked_point,
 
 static void
 generalization_move_handle(Generalization *genlz, Handle *handle,
-		       Point *to, HandleMoveReason reason)
+		       Point *to, HandleMoveReason reason, ModifierKeys modifiers)
 {
   assert(genlz!=NULL);
   assert(handle!=NULL);
@@ -220,8 +222,6 @@ generalization_update_data(Generalization *genlz)
   obj->bounding_box.bottom += GENERALIZATION_WIDTH/2.0 + GENERALIZATION_TRIANGLESIZE;
   obj->bounding_box.right += GENERALIZATION_WIDTH/2.0 + GENERALIZATION_TRIANGLESIZE;
   
-  obj->position = orth->points[0];
-
   /* Calc text pos: */
   num_segm = genlz->orth.numpoints - 1;
   points = genlz->orth.points;
@@ -257,6 +257,49 @@ generalization_update_data(Generalization *genlz)
   rectangle_union(&obj->bounding_box, &rect);
 }
 
+static ObjectChange *
+generalization_add_segment_callback(Object *obj, Point *clicked, gpointer data)
+{
+  ObjectChange *change;
+  change = orthconn_add_segment((OrthConn *)obj, clicked);
+  generalization_update_data((Generalization *)obj);
+  return change;
+}
+
+static ObjectChange *
+generalization_delete_segment_callback(Object *obj, Point *clicked, gpointer data)
+{
+  ObjectChange *change;
+  change = orthconn_delete_segment((OrthConn *)obj, clicked);
+  generalization_update_data((Generalization *)obj);
+  return change;
+}
+
+
+static DiaMenuItem object_menu_items[] = {
+  { N_("Add segment"), generalization_add_segment_callback, NULL, 1 },
+  { N_("Delete segment"), generalization_delete_segment_callback, NULL, 1 },
+};
+
+static DiaMenu object_menu = {
+  "Generalization",
+  sizeof(object_menu_items)/sizeof(DiaMenuItem),
+  object_menu_items,
+  NULL
+};
+
+static DiaMenu *
+generalization_get_object_menu(Generalization *genlz, Point *clickedpoint)
+{
+  OrthConn *orth;
+
+  orth = &genlz->orth;
+  /* Set entries sensitive/selected etc here */
+  object_menu_items[0].active = orthconn_can_add_segment(orth, clickedpoint);
+  object_menu_items[1].active = orthconn_can_delete_segment(orth, clickedpoint);
+  return &object_menu;
+}
+
 static Object *
 generalization_create(Point *startpoint,
 	       void *user_data,
@@ -281,17 +324,15 @@ generalization_create(Point *startpoint,
 
   orthconn_init(orth, startpoint);
 
-  generalization_update_data(genlz);
-
   genlz->name = NULL;
   genlz->stereotype = NULL;
-
   genlz->text_width = 0;
-  
   genlz->properties_dialog = NULL;
+
+  generalization_update_data(genlz);
   
-  *handle1 = &orth->endpoint_handles[0];
-  *handle2 = &orth->endpoint_handles[1];
+  *handle1 = orth->handles[0];
+  *handle2 = orth->handles[orth->numpoints-2];
 
   return (Object *)genlz;
 }
@@ -332,20 +373,68 @@ generalization_copy(Generalization *genlz)
   return (Object *)newgenlz;
 }
 
+static void
+generalization_state_free(ObjectState *ostate)
+{
+  GeneralizationState *state = (GeneralizationState *)ostate;
+  g_free(state->name);
+  g_free(state->stereotype);
+}
+
+static GeneralizationState *
+generalization_get_state(Generalization *genlz)
+{
+  GeneralizationState *state = g_new(GeneralizationState, 1);
+
+  state->obj_state.free = generalization_state_free;
+
+  state->name = g_strdup(genlz->name);
+  state->stereotype = g_strdup(genlz->stereotype);
+  
+  return state;
+}
 
 static void
-generalization_save(Generalization *genlz, int fd)
+generalization_set_state(Generalization *genlz, GeneralizationState *state)
 {
-  orthconn_save(&genlz->orth, fd);
+  g_free(genlz->name);
+  g_free(genlz->stereotype);
+  genlz->name = state->name;
+  genlz->stereotype = state->stereotype;
+  
+  genlz->text_width = 0.0;
+  if (genlz->name != NULL) {
+    genlz->text_width =
+      font_string_width(genlz->name, genlz_font, GENERALIZATION_FONTHEIGHT);
+  }
+  if (genlz->stereotype != NULL) {
+    genlz->text_width = MAX(genlz->text_width,
+			  font_string_width(genlz->stereotype, genlz_font, GENERALIZATION_FONTHEIGHT));
+  }
+  
+  g_free(state);
+  
+  generalization_update_data(genlz);
+}
 
-  write_string(fd, genlz->name);
-  write_string(fd, genlz->stereotype);
+static void
+generalization_save(Generalization *genlz, ObjectNode obj_node,
+		    const char *filename)
+{
+  orthconn_save(&genlz->orth, obj_node);
+
+  data_add_string(new_attribute(obj_node, "name"),
+		  genlz->name);
+  data_add_string(new_attribute(obj_node, "stereotype"),
+		  genlz->stereotype);
 }
 
 static Object *
-generalization_load(int fd, int version)
+generalization_load(ObjectNode obj_node, int version,
+		    const char *filename)
 {
   Generalization *genlz;
+  AttributeNode attr;
   OrthConn *orth;
   Object *obj;
 
@@ -361,10 +450,17 @@ generalization_load(int fd, int version)
   obj->type = &generalization_type;
   obj->ops = &generalization_ops;
 
-  orthconn_load(orth, fd);
+  orthconn_load(orth, obj_node);
+
+  genlz->name = NULL;
+  attr = object_find_attribute(obj_node, "name");
+  if (attr != NULL)
+    genlz->name = data_string(attribute_first_data(attr));
   
-  genlz->name = read_string(fd);
-  genlz->stereotype = read_string(fd);
+  genlz->stereotype = NULL;
+  attr = object_find_attribute(obj_node, "stereotype");
+  if (attr != NULL)
+    genlz->stereotype = data_string(attribute_first_data(attr));
 
   genlz->text_width = 0.0;
 
@@ -384,19 +480,16 @@ generalization_load(int fd, int version)
   return (Object *)genlz;
 }
 
-static void
-apply_callback(GtkWidget *widget, gpointer data)
+static ObjectChange *
+generalization_apply_properties(Generalization *genlz)
 {
-  Generalization *genlz;
   GeneralizationPropertiesDialog *prop_dialog;
+  ObjectState *old_state;
   char *str;
 
-  genlz = (Generalization *)data;
   prop_dialog = genlz->properties_dialog;
 
-  (prop_dialog->changed_callback)((Object *)genlz,
-				  prop_dialog->changed_callback_data,
-				  BEFORE_CHANGE);
+  old_state = (ObjectState *)generalization_get_state(genlz);
 
   /* Read from dialog and put in object: */
   if (genlz->name != NULL)
@@ -435,10 +528,9 @@ apply_callback(GtkWidget *widget, gpointer data)
   }
   
   generalization_update_data(genlz);
-  
-  (prop_dialog->changed_callback)((Object *)genlz,
-				  prop_dialog->changed_callback_data,
-				  AFTER_CHANGE);
+  return new_object_state_change((Object *)genlz, old_state, 
+				 (GetStateFunc)generalization_get_state,
+				 (SetStateFunc)generalization_set_state);
 }
 
 static void
@@ -466,14 +558,11 @@ fill_in_dialog(Generalization *genlz)
   }
 }
 
-static void
-generalization_show_properties(Generalization *genlz,
-			   ObjectChangedFunc *changed_callback,
-			   void *changed_callback_data)
+static GtkWidget *
+generalization_get_properties(Generalization *genlz)
 {
   GeneralizationPropertiesDialog *prop_dialog;
   GtkWidget *dialog;
-  GtkWidget *button;
   GtkWidget *entry;
   GtkWidget *hbox;
   GtkWidget *label;
@@ -483,64 +572,38 @@ generalization_show_properties(Generalization *genlz,
     prop_dialog = g_new(GeneralizationPropertiesDialog, 1);
     genlz->properties_dialog = prop_dialog;
 
-    prop_dialog->changed_callback = changed_callback;
-    prop_dialog->changed_callback_data = changed_callback_data;
-    
-    dialog = gtk_dialog_new();
+    dialog = gtk_vbox_new(FALSE, 0);
     prop_dialog->dialog = dialog;
     
-    gtk_signal_connect (GTK_OBJECT (dialog), "delete_event",
-			GTK_SIGNAL_FUNC(gtk_widget_hide), NULL);
-    
-    gtk_window_set_title (GTK_WINDOW (dialog), "Generalization properties");
-    gtk_container_border_width (GTK_CONTAINER (dialog), 5);
-
     hbox = gtk_hbox_new(FALSE, 5);
 
-    label = gtk_label_new("Name:");
+    label = gtk_label_new(_("Name:"));
     gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
     entry = gtk_entry_new();
     prop_dialog->name = GTK_ENTRY(entry);
     gtk_box_pack_start (GTK_BOX (hbox), entry, TRUE, TRUE, 0);
     gtk_widget_show (label);
     gtk_widget_show (entry);
-    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), 
-			hbox, TRUE, TRUE, 0);
+    gtk_box_pack_start (GTK_BOX (dialog), hbox, TRUE, TRUE, 0);
     gtk_widget_show(hbox);
 
     hbox = gtk_hbox_new(FALSE, 5);
-    gtk_container_border_width (GTK_CONTAINER (hbox), 5);
-    label = gtk_label_new("Stereotype:");
+    gtk_container_set_border_width (GTK_CONTAINER (hbox), 5);
+    label = gtk_label_new(_("Stereotype:"));
     gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
     entry = gtk_entry_new();
     prop_dialog->stereotype = GTK_ENTRY(entry);
     gtk_box_pack_start (GTK_BOX (hbox), entry, TRUE, TRUE, 0);
     gtk_widget_show (label);
     gtk_widget_show (entry);
-    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), 
-			hbox, TRUE, TRUE, 0);
+    gtk_box_pack_start (GTK_BOX (dialog), hbox, TRUE, TRUE, 0);
     gtk_widget_show(hbox);
-    
-
-    button = gtk_button_new_with_label ("Apply");
-    GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
-    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area), 
-			button, TRUE, TRUE, 0);
-    gtk_signal_connect (GTK_OBJECT (button), "clicked",
-			GTK_SIGNAL_FUNC(apply_callback),
-			genlz);
-    gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
-			GTK_SIGNAL_FUNC(gtk_widget_hide),
-			GTK_OBJECT(dialog));
-    gtk_widget_grab_default (button);
-    gtk_widget_show (button);
-  } else {
-    genlz->properties_dialog->changed_callback = changed_callback;
-    genlz->properties_dialog->changed_callback_data = changed_callback_data;
-
   }
+  
   fill_in_dialog(genlz);
   gtk_widget_show (genlz->properties_dialog->dialog);
+
+  return genlz->properties_dialog->dialog;
 }
 
 
